/**
 * Copyright (c) 2016-2019 人人开源 All rights reserved.
 *
 * https://www.renren.io
 *
 * 版权所有，侵权必究！
 */

package io.renren.modules.sys.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.renren.common.utils.R;
import io.renren.modules.sys.dao.SysUserTokenDao;
import io.renren.modules.sys.entity.SysUserTokenEntity;
import io.renren.modules.sys.oauth2.TokenGenerator;
import io.renren.modules.sys.service.SysUserTokenService;
import io.renren.modules.sys.service.alg.SysLoginAlgService;
import io.renren.modules.sys.service.bigdata.SysLoginBigDataService;
import io.renren.modules.sys.service.cloud.SysLoginCloudService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;


@Service("sysUserTokenService")
public class SysUserTokenServiceImpl extends ServiceImpl<SysUserTokenDao, SysUserTokenEntity> implements SysUserTokenService {
	//12小时后过期
	private final static int EXPIRE = 3600 * 12;

	@Autowired
	private SysLoginAlgService sysLoginAlgService;
	@Autowired
	private SysLoginCloudService sysLoginCloudService;
	@Autowired
	private SysLoginBigDataService sysLoginBigDataService;


	@Override
	public R createToken(long userId) {
		//生成一个token
		String token = TokenGenerator.generateValue();

		//当前时间
		Date now = new Date();
		//过期时间
		Date expireTime = new Date(now.getTime() + EXPIRE * 1000);

		//判断是否生成过token
		SysUserTokenEntity tokenEntity = this.getById(userId);
		if(tokenEntity == null){
			tokenEntity = new SysUserTokenEntity();
			tokenEntity.setUserId(userId);
			tokenEntity.setToken(token);
			tokenEntity.setUpdateTime(now);
			tokenEntity.setExpireTime(expireTime);


			sysLoginAlgService.insertTokenToAlg(tokenEntity);
			sysLoginBigDataService.insertTokenToAlg(tokenEntity);
			sysLoginCloudService.insertTokenToAlg(tokenEntity);

//			//算法平台新增token
//			sysLoginAlgService.save(tokenEntity);
//			//云平台新增token
//			sysLoginCloudService.save(tokenEntity);
//			//大数据平台新增token
//			sysLoginBigDataService.save(tokenEntity);

			//保存token
			this.save(tokenEntity);

		}else{
			tokenEntity.setToken(token);
			tokenEntity.setUpdateTime(now);
			tokenEntity.setExpireTime(expireTime);

			//更新token
			this.updateById(tokenEntity);

			//更新算法平台token
			sysLoginAlgService.updataTokenToAlg(tokenEntity);
			//更新云平台token
			sysLoginCloudService.updataTokenToCloud(tokenEntity);
			//跟新大数据平台token
			sysLoginBigDataService.updataTokenToBigData(tokenEntity);
		}


		R r = R.ok().put("token", token).put("expire", EXPIRE);

		return r;
	}

	@Override
	public void logout(long userId) {
		//生成一个token
		String token = TokenGenerator.generateValue();

		//修改token
		SysUserTokenEntity tokenEntity = new SysUserTokenEntity();
		tokenEntity.setUserId(userId);
		tokenEntity.setToken(token);

		sysLoginAlgService.logOutPlatForm(tokenEntity);
		sysLoginCloudService.logOutPlatForm(tokenEntity);
		sysLoginBigDataService.logOutPlatForm(tokenEntity);
		this.updateById(tokenEntity);
	}
}
